/*------------------------------------------------------------------------------
 lib/djv_base/box3_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv {
namespace box {

#define _BOX3_INIT \
  x(Box<T, 3>::position.e[0]), \
  y(Box<T, 3>::position.e[1]), \
  z(Box<T, 3>::position.e[2]), \
  w(Box<T, 3>::size.e[0]), \
  h(Box<T, 3>::size.e[1]), \
  d(Box<T, 3>::size.e[2])

template<typename T>
inline Box3<T>::Box3() : _BOX3_INIT {}

template<typename T>
inline Box3<T>::Box3(const Box3<T> & in) : Box<T, 3>(in), _BOX3_INIT {}

template<typename T>
inline Box3<T>::Box3(const Box<T, 3> & in) : Box<T, 3>(in), _BOX3_INIT {}

template<typename T>
inline Box3<T>::Box3(T x, T y, T z, T w, T h, T d) :
  Box<T, 3>(Vec3<T>(x, y, z), Vec3<T>(w, h, d)), _BOX3_INIT {}

template<typename T>
inline Box3<T>::Box3(T w, T h, T d) : Box<T, 3>(Vec3<T>(w, h, d)), _BOX3_INIT {}

template<typename T>
inline Box3<T>::Box3(const Vec3<T> & position, const Vec3<T> & size) :
  Box<T, 3>(position, size), _BOX3_INIT {}

template<typename T>
inline Box3<T>::Box3(const Vec3<T> & size) : Box<T, 3>(size), _BOX3_INIT {}

template<typename T>
inline void Box3<T>::x1(const T & in) { w = in - x; }
template<typename T>
inline void Box3<T>::y1(const T & in) { h = in - y; }
template<typename T>
inline void Box3<T>::z1(const T & in) { d = in - z; }

template<>
inline void Box3<int>::x1(const int & in) { w = in - x + 1; }
template<>
inline void Box3<int>::y1(const int & in) { h = in - y + 1; }
template<>
inline void Box3<int>::z1(const int & in) { d = in - z + 1; }

template<typename T>
inline T Box3<T>::x1() const { return x + w; }
template<typename T>
inline T Box3<T>::y1() const { return y + h; }
template<typename T>
inline T Box3<T>::z1() const { return z + d; }

template<>
inline int Box3<int>::x1() const { return x + w - 1; }
template<>
inline int Box3<int>::y1() const { return y + h - 1; }
template<>
inline int Box3<int>::z1() const { return z + d - 1; }

template<typename T>
inline void Box3<T>::position1(const Vec3<T> & in)
{
  w = in.x - x;
  h = in.y - y;
  d = in.z - z;
}

template<typename T>
inline Vec3<T> Box3<T>::position1() const
{
  return Vec3<T>(x + w, y + h, z + d);
}

template<>
inline void Box3<int>::position1(const Vec3<int> & in)
{
  w = in.x - x + 1;
  h = in.y - y + 1;
  d = in.z - z + 1;
}

template<>
inline Vec3<int> Box3<int>::position1() const
{
  return Vec3<int>(x + w - 1, y + h - 1, z + d - 1);
}

}
}

