/*------------------------------------------------------------------------------
 lib/djv_base/mem.h
 
 Memory

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_MEM_H
#define DJV_MEM_H

#include <djv_base/list.h>

namespace djv {
namespace mem {

// Constants.

const uint64_t kilobyte = 1024;
const uint64_t megabyte = kilobyte * 1024;
const uint64_t gigabyte = megabyte * 1024;
const uint64_t terabyte = gigabyte * 1024;

// Functions.

void copy(const void *, void *, size_t size);
void set(uint8_t, void *, size_t size);
void zero(void *, size_t size);
int  compare(const void *, const void *, size_t size);

// Endian.

enum ENDIAN { MSB, LSB };

#if defined(DJV_MSB)
static const ENDIAN endian_ = MSB;
static const ENDIAN _endian = LSB;
#else
static const ENDIAN endian_ = LSB;
static const ENDIAN _endian = MSB;
#endif
inline ENDIAN endian() { return endian_; }

inline void endian(void *, size_t size, size_t word_size);
inline void endian(const void *, void *, size_t size, size_t word_size);

// Memory buffer.

template<typename T>
class Buffer {
public:

  inline Buffer();
  inline Buffer(const Buffer &);
  inline Buffer(size_t);
  inline ~Buffer();
  
  inline Buffer & operator = (const Buffer &);
  
  inline void size(size_t);
  size_t size() const { return _size; }
  
  T * get() { return _data; }
  T * operator () () { return _data; }
  const T * get() const { return _data; }
  const T * operator () () const { return _data; }
  
  inline void del();

  inline void zero();

private:

  T * _data;
  size_t _size;
};

// Labels.

const List<Str> & endian_label();

}
}

#include <djv_base/mem_.h>

#endif

