/*------------------------------------------------------------------------------
 lib/djv_base/mtx.h
 
 Matrices

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_MTX_H
#define DJV_MTX_H

#include <djv_base/vec.h>

namespace djv {
namespace mtx {

// Base matrix.

template<typename T, int E>
class Mtx {
public:

  Mtx() { identity(); }
  Mtx(const Mtx<T, E> & in) { set(in); }
  Mtx(const T * in) { set(in); }
  virtual ~Mtx() {}

  T e [E * E];
  
  void set(const Mtx<T, E> &);
  void set(const T *);
  void zero();
  void identity();
};

// 3x3 matrix.

template<typename T>
class Mtx3 : public Mtx<T, 3> {
public:

  Mtx3() {}
  Mtx3(const Mtx3<T> & in) : Mtx<T, 3>(in) {}
  Mtx3(const Mtx<T, 3> & in) : Mtx<T, 3>(in) {}
  inline Mtx3(T, T, T, T, T, T, T, T, T);
  Mtx3(const T * in) : Mtx<T, 3>(in) {}
  
  inline void set(T, T, T, T, T, T, T, T, T);
};

// 4x4 matrix.

template<typename T>
class Mtx4 : public Mtx<T, 4> {
public:

  Mtx4() {}
  Mtx4(const Mtx4<T> & in) : Mtx<T, 4>(in) {}
  Mtx4(const Mtx<T, 4> & in) : Mtx<T, 4>(in) {}
  inline Mtx4(T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T);
  Mtx4(const T * in) : Mtx<T, 4>(in) {}
  
  inline void set(T, T, T, T, T, T, T, T, T, T, T, T, T, T, T, T);
};

// Functions.

template<typename T, int E>
inline Mtx<T, E> transpose(const Mtx<T, E> &);

// Operators.

template<typename T, int E>
inline Mtx<T, E> operator * (const Mtx<T, E> &, const Mtx<T, E> &);

// Comparison.

template<typename T, int E>
inline bool operator == (const Mtx<T, E> &, const Mtx<T, E> &);
template<typename T, int E>
inline bool operator != (const Mtx<T, E> &, const Mtx<T, E> &);

// Conversion.

template<typename T, int E>
inline bool serialize(List<Str> *, Mtx<T, E> *);
template<typename T, int E>
inline List<Str> serialize(const Mtx<T, E> &);

// Debugging.

template<typename T, int E>
inline Dbg & operator << (Dbg &, const Mtx<T, E> &);

}
}

#include <djv_base/mtx_.h>
#include <djv_base/mtx_fnc_.h>

#endif

