/*------------------------------------------------------------------------------
 lib/djv_base/str_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv {
namespace str {

inline Str lower(const Str & in)
{ 
  Str out;
  out.resize(in.size());
  for (uint i = 0; i < in.size(); ++i)
    out[i] = (in[i] >= 'A' && in[i] <= 'Z') ? (in[i] + 'a' - 'A') : in[i];
  return out;
}

inline Str upper(const Str & in)
{
  Str out;
  out.resize(in.size());
  for (uint i = 0; i < in.size(); ++i)
    out[i] = (in[i] >= 'a' && in[i] <= 'z') ? (in[i] - 'a' - 'A') : in[i];
  return out;
}

inline bool is_digit(char in)
{
  return in >= '0' && in <= '9';
}

inline Str join(const List<Str> & in, const Str & separator)
{
  Str out;
  for (uint i = 0; i < in.size(); ++i) {
    if (i) out += separator;
    out += in[i];
  }
  return out;
}

inline int str_to_int(const char * in, int max_len)
{
  bool negative = false;
  if (*in)
    if ('-' == *in) {
      negative = true;
      ++in;
    }
  
  int out = 0;
  while (*in && max_len--) {
    out *= 10;
    out += *in++ - '0';
  }
  return negative ? -out : out;
}

inline int str_to_int(const Str & in)
{
  return str_to_int(in.c_str());
}

inline bool serialize(List<Str> * in, Str * out)
{
  if (! in->size()) return false;
  *out = in->front();
  in->pop_front();
  return true;
}

inline List<Str> serialize(const Str & in)
  { return List<Str>() << in; }

inline bool serialize(List<Str> * in, bool * out)
  { return serialize(in, out, bool_label()); }

inline List<Str> serialize(bool in)
  { return List<Str>() << bool_label()[in]; }

inline bool serialize(List<Str> * in, int * out)
{
  if (! in->size()) return false;
  *out = str_to_int(in->front());
  in->pop_front();
  return true;
}

inline List<Str> serialize(int in)
  { return List<Str>() << int_to_str(in); }

inline bool serialize(List<Str> * in, double * out)
{
  if (! in->size()) return false;
  *out = str_to_float(in->front());
  in->pop_front();
  return true;
}

inline List<Str> serialize(double in)
  { return List<Str>() << float_to_str(in); }

template <typename T>
inline bool serialize(List<Str> * in, T * out, const List<Str> & label)
{
  if (! in->size()) return false;
  
  // XXX Case-insensitive?
  
  //const int i = list::find(in->front(), label);
  //if (-1 == i) return false;
  
  uint i = 0;
  for (; i < label.size(); ++i)
    if (compare(in->front(), label[i])) break;
  if (label.size() == i) return false;
  
  in->pop_front();
  *out = T(i);
  return true;
}

template <typename T>
inline List<Str> serialize(const T & in, const List<Str> & label)
  { return List<Str>() << label[in]; }

inline bool serialize(List<Str> * in, List<Str> * out)
{
  *out = *in;
  in->clear();
  return true;
}

inline List<Str> serialize(const List<Str> & in) { return in; }

inline bool serialize(List<Str> * in, List<int> * out)
{
  out->resize(in->size());
  for (uint i = 0; i < in->size(); ++i)
    (*out)[i] = str_to_int((*in)[i]);
  in->clear();
  return true;
}

inline List<Str> serialize(const List<int> & in)
{
  List<Str> out;
  out.resize(in.size());
  for (uint i = 0; i < in.size(); ++i)
    out[i] = int_to_str(in[i]);
  return out;
}

}
}
