/*------------------------------------------------------------------------------
 lib/djv_image/color.h
 
 Color
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_COLOR_H
#define DJV_IMAGE_COLOR_H

#include <djv_image/pixel.h>

namespace djv {
namespace image {
namespace color {

class Color {
public:

  Color();
  Color(const Color &);
  Color(pixel::PIXEL);
  Color(pixel::F32_T l);
  Color(pixel::F32_T l, pixel::F32_T a);
  Color(pixel::F32_T r, pixel::F32_T g, pixel::F32_T b);
  Color(pixel::F32_T r, pixel::F32_T g, pixel::F32_T b, pixel::F32_T a);

  Color & operator = (const Color &);

  // Value.
  
  inline void set(pixel::PIXEL);
  pixel::PIXEL pixel() const { return _pixel; }
  int channels() const { return _channels; }
  
  // Components.
  
  inline void set_u8(pixel::U8_T, int = -1);
  inline void set_u10(pixel::U10_T, int = -1);
  inline void set_u16(pixel::U16_T, int = -1);
  inline void set_f16(pixel::F16_T, int = -1);
  inline void set_f32(pixel::F32_T, int = -1);
  
  inline pixel::U8_T get_u8(int) const;
  inline pixel::U10_T get_u10(int) const;
  inline pixel::U16_T get_u16(int) const;
  inline pixel::F16_T get_f16(int) const;
  inline pixel::F32_T get_f32(int) const;

  // Functions.
  
  void zero();
  
  // Data.

  uint8_t * data() { return _data; }
  const uint8_t * data() const  { return _data; }

private:

  void init();

  pixel::PIXEL _pixel;
  int _channels;
  uint8_t _data [pixel::channels_max * pixel::bytes_max];
};

// Conversion.

void convert(const Color &, Color *);

inline void rgb_to_hsv(const double [3], double [3]);
inline void hsv_to_rgb(const double [3], double [3]);

bool serialize(List<Str> *, Color *);
List<Str> serialize(const Color &);

// Comparison.

bool operator == (const Color &, const Color &);
bool operator != (const Color &, const Color &);

// Labels.

Str label(const Color &);

// Debugging.

Dbg & operator << (Dbg &, const Color &);

}
}
}

#include <djv_image/color_.h>

#endif

