/*------------------------------------------------------------------------------
 lib/djv_image/gl.h
 
 OpenGL

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_GL_H
#define DJV_IMAGE_GL_H

#include <djv_image/pixel.h>

namespace djv {
namespace image {
namespace gl {

// Functions.

void ortho(int width, int height, double min = -1.0, double max = 1.0);

GLenum format(pixel::PIXEL, bool bgr = false);
GLenum type(pixel::PIXEL);

// Debugging.
//
// References:
// * Dave Shreiner, "Performance OpenGL: Platform Independent Techniques",
//   SIGGRAPH 2001.

#if defined(DJV_DBG)
#define DJV_DBG_GL(cmd) \
  cmd; \
  { \
    ::GLenum error; \
    if ((error = ::glGetError()) != GL_NO_ERROR) \
      DJV_DBG_PRN(djv::str::Format("%1 = %2 (%3, line %4)"). \
        arg(#cmd).arg((char *)::gluErrorString(error)). \
        arg(__FILE__).arg(__LINE__)); \
  }
#else
#define DJV_DBG_GL(cmd) cmd;
#endif

}
}
}

#endif

