/*------------------------------------------------------------------------------
 lib/djv_image/gl_data_copy.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/gl_data_copy.cxx"

#include "gl_data.h"
#include <djv_image/gl_pbuffer.h>

namespace djv {
namespace image {
namespace gl {
namespace data {

void copy(const image::Data & in, image::Data * out, const Options & options)
  throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "copy");
  DJV_DBG_PRN("in = " << in);

  pbuffer::Pbuffer pbuffer(out->info().size);
  Data data(&in);
  copy(data, pbuffer, out, options);
}

void copy(const Data & in, void * context_share, image::Data * out,
  const Options & options
) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "copy");
  if (out) DJV_DBG_PRN("out = " << *out);

  pbuffer::Pbuffer pbuffer(out ? out->info().size : in.get()->info().size,
    context_share);
  copy(in, pbuffer, out, options);
}

void copy(const Data & in, const pbuffer::Pbuffer & pbuffer, image::Data * out,
  const Options & options
) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "copy");
  if (out) DJV_DBG_PRN("out = " << *out);
  DJV_DBG_PRN("channel = " << options.channel);

  Options _options = options;
  if (out) {
    if (out->info().mirror_x) _options.mirror_x = ! _options.mirror_x;
    if (out->info().mirror_y) _options.mirror_y = ! _options.mirror_y;
  }
  
  //DJV_DBG_GL(glClearColor(0, 1, 0, 0));
  djv::image::Color background(djv::image::pixel::RGB_F32);
  djv::image::color::convert(options.background, &background);
  DJV_DBG_GL(glClearColor(
    background.get_f32(0), background.get_f32(1), background.get_f32(2), 0.0));
  
  const Vec2i & size = out ? out->info().size : in.get()->info().size;
  const Vec2i & pbuffer_size = pbuffer.size();
  for (int y = 0; y < size.y; y += pbuffer_size.y)
    for (int x = 0; x < size.x; x += pbuffer_size.x) {

      const Box2i tile(x, y,
        math::min(pbuffer_size.x, size.x - x),
        math::min(pbuffer_size.y, size.y - y));
      DJV_DBG_PRN("tile = " << tile);
      
      DJV_DBG_GL(glMatrixMode(GL_PROJECTION));
      DJV_DBG_GL(glLoadIdentity());
      DJV_DBG_GL(glOrtho(tile.x, tile.x + tile.w, tile.y, tile.y + tile.h,
        -1.0, 1.0));
      DJV_DBG_GL(glMatrixMode(GL_MODELVIEW));
      DJV_DBG_GL(glLoadIdentity());
      
      DJV_DBG_GL(glViewport(0, 0, tile.w, tile.h));
      DJV_DBG_GL(glClear(GL_COLOR_BUFFER_BIT));
      
      in.draw(_options);
      if (out) in.read(out, tile);
    }
}

}
}
}
}

