/*------------------------------------------------------------------------------
 lib/djv_image/io_cineon_load.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/io_cineon_load.cxx"

#include "io_cineon.h"

namespace djv {
namespace image {
namespace io {
namespace cineon {

Str Load::help() const
{
  return str::Format(
"%1:\n"
"* color (value) - Input color space: %2. Default: %3.\n"
"* print (black) (white) (gamma) (soft clip) - Print values. Default: "
"%4 %5 %6 %7.\n"
  ).
    arg(plugin_name).
    arg(str::join(dpx::color_label(), ", ")).
    arg(dpx::color_label()[_options.color]).
    arg(_options.print.black).
    arg(_options.print.white).
    arg(_options.print.gamma).
    arg(_options.print.soft_clip);
}

Plugin * Load::copy() const
  { Load * out = new Load; out->_options = _options; return out; }

bool Load::option(const Str & in, List<Str> * data)
{
  _print_lut.reset();

  if (str::compare(in, option_list()[OPTION_COLOR]))
    return str::serialize(data, &_options.color, dpx::color_label());
  else if (str::compare(in, option_list()[OPTION_PRINT]))
    return dpx::serialize(data, &_options.print);
  return false;
}

List<Str> Load::option(const Str & in) const
{
  if (str::compare(in, option_list()[OPTION_COLOR]))
    return str::serialize(_options.color, dpx::color_label());
  else if (str::compare(in, option_list()[OPTION_PRINT]))
    return dpx::serialize(_options.print);
  return List<Str>();
}

const List<Str> & Load::option_list() const { return load_option_label(); }

void Load::open(const File & in, Info * info) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::open");
  DJV_DBG_PRN("in = " << in);

  _file = in;
  _open(_file.get(_file.seq.start), info);

  if (file::SEQ == _file.type) {
    info->time.start = _file.seq.start;
    info->time.end = _file.seq.end;
  }
}

void Load::_open(const File & in, Info * info) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::_open");
  DJV_DBG_PRN("in = " << in);

  _io.open(in, djv::io::READ);
  _print = false;
  header_get(&_io, info, &_print);
  DJV_DBG_PRN("info " << *info);
}

const Image * Load::get(int frame, int proxy) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Load::get");
  DJV_DBG_PRN("frame = " << frame);
  DJV_DBG_PRN("proxy = " << proxy);

  profile::Profile profile;
  if (dpx::COLOR_PRINT == _options.color ||
    (dpx::COLOR_AUTO == _options.color && _print)
  ) {
    DJV_DBG_PRN("profile");
    profile.type = profile::LUT;
    if (! _print_lut.get()) _print_lut = std::auto_ptr<Data>(
      new Data(dpx::unprint_lut(_options.print)));
    profile.lut = *_print_lut;
  }
  
  // Open.
  
  const File file = File(_file.get(frame != -1 ? frame : _file.seq.start));
  DJV_DBG_PRN("file = " << file);
  
  const Cache cache(file, frame, proxy);
  if (cache == _cache) {
    DJV_DBG_PRN("cached");
    _image.profile(profile);
    return &_image;
  }
  _cache = cache;
  
  Info info;
  info.profile = profile;
  _open(file, &info);

  // Read.
  
  const uint8_t * p = _io.mmap_p();
  _io.position(data::data_bytes(info), djv::io::OFFSET);  
  if (! proxy) _image.set(info, p);
  else {
    _image_tmp.set(info, p);
    info.size = data::proxy_scale(info.size, proxy);
    info.proxy = proxy;
    _image.set(info);
    data::proxy_scale(_image_tmp, &_image, proxy);
  }
  DJV_DBG_PRN("image = " << _image);
  return &_image;
}

}
}
}
}
