/*------------------------------------------------------------------------------
 lib/djv_image/io_cineon_save.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/io_cineon_save.cxx"

#include "io_cineon.h"

namespace djv {
namespace image {
namespace io {
namespace cineon {

Str Save::help() const
{
  return str::Format(
"%1:\n"
"* color (value) - Output color space: %2. Default: %3.\n"
"* print (black) (white) (gamma) - Print values. Default: %4 %5 %6.\n"
  ).
    arg(plugin_name).
    arg(str::join(dpx::color_label(), ", ")).
    arg(dpx::color_label()[_options.color]).
    arg(_options.print.black).
    arg(_options.print.white).
    arg(_options.print.gamma);
}

Plugin * Save::copy() const
  { Save * out = new Save; out->_options = _options; return out; }

bool Save::option(const Str & in, List<Str> * data)
{
  if (str::compare(in, option_list()[OPTION_COLOR]))
    return str::serialize(data, &_options.color, dpx::color_label());
  else if (str::compare(in, option_list()[OPTION_PRINT]))
    return dpx::serialize(data, &_options.print);
  return false;
}

List<Str> Save::option(const Str & in) const
{
  if (str::compare(in, option_list()[OPTION_COLOR]))
    return str::serialize(_options.color, dpx::color_label());
  else if (str::compare(in, option_list()[OPTION_PRINT]))
    return dpx::serialize(_options.print);
  return List<Str>();
}

const List<Str> & Save::option_list() const { return save_option_label(); }

void Save::open(const File & in, const Info & info) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::open");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("info = " << info);
  
  _file = in;
  
  _info = Info();
  _info.size = info.size;
  _info.mirror_y = true;
  _info.pixel = pixel::RGB_U10;
  _info.endian = mem::MSB;
  _info.align = 4;
  
  if (dpx::COLOR_PRINT == _options.color || dpx::COLOR_AUTO == _options.color) {
    DJV_DBG_PRN("print");
    _info.profile.type = profile::LUT;
    _info.profile.lut = dpx::print_lut(_options.print);
  }
  
  _image.set(_info);
  DJV_DBG_PRN("image = " << _image);
}

void Save::_open(const File & in, const tag::Tag & tag) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::_open");
  DJV_DBG_PRN("in = " << in);

  _io.open(in, djv::io::WRITE);
  image::Info info = _image.info();
  info.tag = tag;
  header_set(&_io, in, info, _options.color);
}

void Save::set(const Image & in, int frame) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::set");
  DJV_DBG_PRN("in = " << in);
  
  const Image * image_p = &in;
  if (in.info() != _image.info() || _info.profile.type != profile::RAW) {
    DJV_DBG_PRN("convert = " << _image);
    _image.zero();
    gl::data::Options options;
    options.profile = _info.profile;
    gl::data::copy(in, &_image, options);
    image_p = &_image;
  }
  
  _open(_file.get(frame), in.info().tag);
  _io.set(image_p->data(), image_p->data_bytes());
  header_update(&_io);
  _io.close();
}

}
}
}
}
