/*------------------------------------------------------------------------------
 lib/djv_image/io_dpx_fnc.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "io_dpx.h"

namespace djv {
namespace image {
namespace io {
namespace dpx {

bool serialize(List<Str> * in, Print * out)
{
  if (! str::serialize(in, &out->black)) return false;
  if (! str::serialize(in, &out->white)) return false;
  if (! str::serialize(in, &out->gamma)) return false;
  return true;
}

bool serialize(List<Str> * in, Unprint * out)
{
  if (! str::serialize(in, &out->black)) return false;
  if (! str::serialize(in, &out->white)) return false;
  if (! str::serialize(in, &out->gamma)) return false;
  if (! str::serialize(in, &out->soft_clip)) return false;
  return true;
}

List<Str> serialize(const Print & in)
{
  return List<Str>() << str::serialize(in.black) << str::serialize(in.white) <<
    str::serialize(in.gamma);
}

List<Str> serialize(const Unprint & in)
{
  return List<Str>() << str::serialize(in.black) << str::serialize(in.white) <<
    str::serialize(in.gamma) << str::serialize(in.soft_clip);
}

bool operator == (const Print & a, const Print & b)
{
  return
    a.black == b.black &&
    a.white == b.white &&
    a.gamma == b.gamma;
}

bool operator == (const Unprint & a, const Unprint & b)
{
  return
    a.black == b.black &&
    a.white == b.white &&
    a.gamma == b.gamma &&
    a.soft_clip == b.soft_clip;
}

const List<Str> & color_label()
{
  static const List<Str> data = List<Str>() << "Auto" << "Raw" << "Print";
  return data;
}

const List<Str> & type_label()
{
  static const List<Str> data = List<Str>() << "Auto" << "U10";
  return data;
}

const List<Str> & endian_label()
{
  static const List<Str> data = List<Str>() << "Auto" << mem::endian_label();
  return data;
}

const List<Str> & tag_label()
{
  static const List<Str> data = List<Str>() <<
    "Source Offset" <<
    "Source Center" <<
    "Source Size" <<
    "Source File" <<
    "Source Time" <<
    "Source Input Device" <<
    "Source Input Serial" <<
    "Source Border" <<
    "Source Pixel Aspect" <<
    "Source Scan Size" <<
    "Film Format" <<
    "Film Frame" <<
    "Film Sequence" <<
    "Film Hold" <<
    "Film Frame Rate" <<
    "Film Shutter" <<
    "Film Frame ID" <<
    "Film Slate" <<
    "TV Interlace" <<
    "TV Field" <<
    "TV Video Signal" <<
    "TV Sample Rate" <<
    "TV Frame Rate" <<
    "TV Time Offset" <<
    "TV Gamma" <<
    "TV Black Level" <<
    "TV Black Gain" <<
    "TV Break Point" <<
    "TV White Level" <<
    "TV Integration Times";
  return data;
}

const List<Str> & load_option_label()
{
  static const List<Str> data = List<Str>() << "Color" << "Print";
  return data;
}

const List<Str> & save_option_label() 
{
  static const List<Str> data = List<Str>() << "Color" << "Print" <<
    "Type" << "Endian";
  return data;
}

}
}
}
}

