/*------------------------------------------------------------------------------
 lib/djv_image/io_ppm.h
 
 NetPBM Image I/O

 References:
 * Netpbm, "PPM Format Specification",
   http://netpbm.sourceforge.net/doc/ppm.html

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_PPM_H
#define DJV_IMAGE_IO_PPM_H

#include <djv_image/io.h>

namespace djv {
namespace image {
namespace io {
namespace ppm {

// Information.

static const Str plugin_name = "PPM";
static const List<Str> extension_list = List<Str>() << ".ppm" << ".pnm" <<
  ".pgm" << ".pbm";

// Options.

enum TYPE { TYPE_AUTO, U1 };
enum MODE { ASCII, BINARY };

// Load.

class Load : public io::Load {
public:

  Load();
  
  Str name() const { return ppm::plugin_name; }
  const List<Str> & extension_list() const { return ppm::extension_list; }
  
  Plugin * copy() const { return new Load; }
  
  void open(const File &, io::Info *) throw (Error);
  const Image * get(int frame, int proxy) throw (Error);

private:

  void _open(const File &, io::Info *) throw (Error);

  File _file;
  djv::Io _io;
  int _bit_depth;
  MODE _mode;
  Image _image, _image_tmp;
};

// Save.

class Save : public io::Save {
public:

  Save();
  
  Str name() const { return ppm::plugin_name; }
  const List<Str> & extension_list() const { return ppm::extension_list; }
  Str help() const;
  
  Plugin * copy() const;
  
  struct Options {
    Options() : type(TYPE_AUTO), mode(BINARY) {}
    TYPE type;
    MODE mode;
  };
  enum OPTION { OPTION_TYPE, OPTION_MODE };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const io::Info &) throw (Error);
  void set(const Image &, int frame) throw (Error);

private:
  
  void _open(const File &) throw (Error);

  Options _options;
  File _file;
  djv::Io _io;
  int _bit_depth;
  Image _image;
};

// Functions.

size_t scanline_bytes(int width, int channels, int bit_depth, MODE);

void ascii_unpack(djv::Io *, void *, int size, int bit_depth) throw (Error);
size_t ascii_pack(const void *, void *, int size, int bit_depth);

// Labels.

const List<Str> & type_label();
const List<Str> & mode_label();
const List<Str> & save_option_label();

}
}
}
}

#endif

