/*------------------------------------------------------------------------------
 lib/djv_image/io_tga_data.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/io_tga_data.cxx"

#include "io_tga.h"

namespace djv {
namespace image {
namespace io {
namespace tga {

void rle_unpack(djv::Io * io, void * out, int size, int channels) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "rle_unpack");
  DJV_DBG_PRN("size = " << size);
  DJV_DBG_PRN("channels = " << channels);
  
  uint8_t * out_p = reinterpret_cast<uint8_t *>(out);
  uint8_t buf [128 * 4]; // Max RLE count * channels.
  
  int i = 0;
  while (i < size) {
  
    // Get RLE information.
    
    uint8_t tmp;
    io->get_u8(&tmp);
    const uint8_t count = (tmp & 0x7f) + 1;
    if (! count || count > (size - i)) return;
    const bool run = (tmp & 0x80) ? true : false;
    const int length = run ? 1 : count;
    DJV_DBG_PRN("length = " << length);
    
    // Unpack pixels.
    
    io->get(buf, length * channels);
    const uint8_t * in_p = buf;
    
    for (int j = 0; j < length * channels; j += channels, in_p += channels)
      switch (channels) {
        case 4: out_p[j + 3] = in_p[3];
        case 3: out_p[j + 2] = in_p[2];
        case 2: out_p[j + 1] = in_p[1];
        case 1: out_p[j + 0] = in_p[0]; break;
      }
      
    if (run)
      for (int j = channels; j < count * channels; j += channels)
        switch (channels) {
          case 4: out_p[j + 3] = out_p[3];
          case 3: out_p[j + 2] = out_p[2];
          case 2: out_p[j + 1] = out_p[1];
          case 1: out_p[j + 0] = out_p[0];
        }

    out_p += count * channels;
    i += count;
  }
  DJV_DBG_PRN("i = " << i);
}

size_t rle_pack(const void * in, void * out, int size, int channels)
{
  DJV_DBG_OBJ2(_DBG, "rle_pack");
  DJV_DBG_PRN("size = " << size);
  DJV_DBG_PRN("channels = " << channels);

  const uint8_t * in_p = reinterpret_cast<const uint8_t *>(in);
  uint8_t * out_p = reinterpret_cast<uint8_t *>(out);
  const uint8_t * const end = in_p + size * channels;
  
  while (in_p < end) {

    // Find pixel runs.
    
    const int max = math::min(0x7f + 1, int(end - in_p) / channels);
    int count = 1;
    for (; count < max; ++count) {
      int c = 0;
      for (; c < channels; ++c)
        if (in_p[count * channels + c] != in_p[(count - 1) * channels + c])
          break;
      if (c < channels) break;
    }
    const bool run = count > 1;
    const int length = run ? 1 : count;
    DJV_DBG_PRN("count = " << count);
    DJV_DBG_PRN("  run = " << run);
    DJV_DBG_PRN("  length = " << length);
    
    // Set RLE information.
    
    *out_p++ = ((count - 1) & 0x7f) | (run << 7);
    
    // Pack pixels.
    
    for (int j = 0; j < length * channels; j += channels)
      switch (channels) {
        case 4: out_p[j + 3] = in_p[j + 3];
        case 3: out_p[j + 2] = in_p[j + 2];
        case 2: out_p[j + 1] = in_p[j + 1];
        case 1: out_p[j    ] = in_p[j    ]; break;
      }
    
    out_p += length * channels;
    in_p += count * channels;
  }
  
  const size_t r = out_p - reinterpret_cast<uint8_t *>(out);
  DJV_DBG_PRN("return = " << r);
  return r;
}

}
}
}
}
