/*------------------------------------------------------------------------------
 lib/djv_image/profile.h

 Image Color Profiles

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_PROFILE_H
#define DJV_IMAGE_PROFILE_H

#include <djv_image/data.h>
#include <djv_image/color.h>

namespace djv {
namespace image {
namespace profile {

enum PROFILE { RAW, LUT, EXPOSURE };

struct Exposure {

  Exposure() : value(0), defog(0), knee_low(0), knee_high(5), gamma(2.2) {}
  
  double value, defog, knee_low, knee_high, gamma;
};

struct Profile {

  Profile() : type(PROFILE(0)) {}
  
  PROFILE type;  
  image::Data lut;
  Exposure exposure;
};

// Comparison.

bool operator == (const Exposure &, const Exposure &);

// Conversion.

bool serialize(List<Str> *, Exposure *);
List<Str> serialize(const Exposure &);

// Labels.

const List<Str> & profile_label();

// Debugging.

Dbg & operator << (Dbg &, PROFILE &);

}
}
}

#endif

