/*------------------------------------------------------------------------------
 lib/djv_ui/choice.h
 
 Choice Widgets

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_CHOICE_H
#define DJV_UI_CHOICE_H

#include <djv_ui/widget.h>
#include <djv_ui/group.h>

namespace djv {
namespace ui {
namespace choice {

// Choice widget.

class Widget : public widget::Widget {
public:

  Widget(const List<Str> & list = List<Str>());
  virtual ~Widget() {}
  
  // Value.
  
  void set(int);
  int get() const;
  Signal<int> sig;

  // List.
  
  void list(const List<Str> &);
  const List<Str> & list() const;
  
  // Label.
  
  void label(widget::Widget *);
  widget::Widget * label() const { return _label; }
  virtual Item * label_align() const { return _label; }
  
private:

  class _Widget;

  DJV_CALLBACK(Widget, widget_callback, int);

  _Widget * _widget;
  widget::Widget * _label;
  layout::Horizontal * _layout;
};

// Editable choice widget.

class Edit : public group::Frame {
public:

  Edit(const List<Str> & list = List<Str>());
  virtual ~Edit() {}
  
  // Value.
  
  void set(const Str &);
  Str get() const;
  Signal<const Str &> sig;

  // List.
  
  void list(const List<Str> &);
  const List<Str> & list() const { return _list; }

  // Widget.
  
  virtual void widget_del();
  
  // Layout.

  virtual void size_str(const Str &);
  
private:

  DJV_CALLBACK(Edit, widget_callback, const Str &);
  DJV_CALLBACK(Edit, menu_callback, int);

  void list_update();

  Str _value;
  List<Str> _list;
  text::Edit * _widget;
  menu::Popup * _menu;
};

}
}
}

#endif
