/*------------------------------------------------------------------------------
 lib/djv_ui/file_browser.h
 
 File Browser Dialog
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_FILE_BROWSER_H
#define DJV_UI_FILE_BROWSER_H

#include <djv_base/file.h>
#include <djv_ui/style.h>
#include <djv_ui/dialog.h>

namespace djv {
namespace ui {
namespace file_browser {

class Browser;

class Dialog : public dialog::Base {
public:

  Dialog();
  
  // Value.
  
  void set(const Str &);
  File get() const;

  void dir(const Str &);
  const Str & dir() const { return _dir; }

  Signal<const File &> sig;
  
  // View.

#if defined(WIN32)
  enum SORT { SORT_NAME, SORT_SIZE, SORT_TIME };
  static const uint sort_size = 3;
#else
  enum SORT { SORT_NAME, SORT_SIZE, SORT_USER, SORT_TIME };
  static const uint sort_size = 4;
#endif

  void images(bool);
  void seq(bool);
  void hidden(bool);
  void sort(SORT);
  void sort_reverse(bool);
  void sort_dirs(bool);
  
  bool images() const { return _images; }
  bool seq() const { return _seq; }
  bool hidden() const { return _hidden; }
  SORT sort() const { return _sort; }
  bool sort_reverse() const { return _sort_reverse; }
  
  // Bookmarks.
  
  void bookmarks(const List<Str> &);
  const List<Str> & bookmarks() const { return _bookmarks; }
  
  // Dialog.
  
  void pin(bool);
  bool pin() const { return _pin; }
  
  // Widget.

  void widget_del();

  // Layout.
  
  void dirty();

  // Debugging.

  Str dbg_name() const { return "djv::ui::file_browser::Dialog"; }
  
private:

  // Callbacks.

  DJV_CALLBACK(Dialog, widget_callback, const Str &);
  DJV_CALLBACK(Dialog, widget_action_callback, const Str &);
  DJV_CALLBACK(Dialog, dir_callback, const Str &);
  DJV_CALLBACK(Dialog, browser_callback, const File &);
  DJV_CALLBACK(Dialog, browser_action_callback, const File &);
  DJV_CALLBACK(Dialog, columns_callback, const List<int> &);
  DJV_CALLBACK(Dialog, columns_action_callback, int);
  
  DJV_CALLBACK(Dialog, up_callback, bool);
  DJV_CALLBACK(Dialog, prev_callback, bool);
  DJV_CALLBACK(Dialog, current_callback, bool);
#if ! defined(WIN32)
  DJV_CALLBACK(Dialog, home_callback, bool);
#endif
  DJV_CALLBACK(Dialog, desktop_callback, bool);
#if defined(WIN32)
  DJV_CALLBACK(Dialog, drive_callback, bool);
#else
  DJV_CALLBACK(Dialog, root_callback, bool);
#endif
  DJV_CALLBACK(Dialog, reload_callback, bool);
  DJV_FL_WIDGET_CALLBACK(Dialog, _up_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _prev_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _recent_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _current_callback);
#if ! defined(WIN32)
  DJV_FL_WIDGET_CALLBACK(Dialog, _home_callback);
#endif
  DJV_FL_WIDGET_CALLBACK(Dialog, _desktop_callback);
#if defined(WIN32)
  DJV_FL_WIDGET_CALLBACK(Dialog, _drive_callback);
#else
  DJV_FL_WIDGET_CALLBACK(Dialog, _root_callback);
#endif
  DJV_FL_WIDGET_CALLBACK(Dialog, _reload_callback);

  DJV_FL_WIDGET_CALLBACK(Dialog, _images_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _seq_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _hidden_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _sort_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _sort_reverse_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _sort_dirs_callback);
  
  DJV_CALLBACK(Dialog, bookmark_del_callback, int);
  DJV_CALLBACK(Dialog, bookmark_del_all_callback, bool);
  DJV_FL_WIDGET_CALLBACK(Dialog, _bookmark_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _bookmark_add_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _bookmark_del_callback);
  DJV_FL_WIDGET_CALLBACK(Dialog, _bookmark_del_all_callback);
  
  DJV_CALLBACK(Dialog, pin_callback, bool);
  DJV_CALLBACK(Dialog, ok_callback, bool);
  DJV_CALLBACK(Dialog, close_callback, bool);
  
  // Update.

  void dir_update();
  void widget_update();
  void menu_update();
  
  // Variables.

  Str _value, _dir, _dir_prev;
  List<Str> _recent;
  bool _images, _seq, _hidden;
  SORT _sort;
  bool _sort_reverse, _sort_dirs;
  List<Str> _bookmarks;
  bool _pin;
  
  menu::Widget * _menu_widget;
  List<int> _menu_recent, _menu_sort, _menu_bookmark;
#if defined(WIN32)
  List<int> _menu_drive;
#endif
  text::Edit * _widget, * _dir_widget;
  Browser * _dir_browser;
  columns::Widget * _columns_widget;
  button::Check * _pin_button;
  style::SIZE _style_size;
};

// Labels.

const List<Str> & sort_label();

// Global.

void set(Callback_Base *, Signal<const File &>::Callback *);
void del(Callback_Base *);

Dialog * dialog();

void del();

}
}
}

#endif

