/*------------------------------------------------------------------------------
 lib/djv_ui/menu_popup.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "menu.h"
#include <djv_ui/style.h>
#include <djv_ui/icon.h>
#include <FL/Fl.H>
#include <FL/Fl_Menu_Button.H>

namespace djv {
namespace ui {
namespace menu {

class Popup::Widget : public Fl_Menu_Button {
public:
  Widget() : Fl_Menu_Button(0, 0, 0, 0) {}
  void draw() {}
};

Popup::Popup(const List<Str> & list) :
  _icon(ui::icon::get("tool_dec"))
{
  frame(FL_UP_BOX);
  
  _widget = new Widget;
  Frame::add(_widget);
  
  size_policy(FIXED, FIXED);
  
  set(list);
  
  _widget->callback(widget_callback, this);
}

void Popup::set(const List<Str> & in)
{
  menu_del();
  add(in);
  menu_end();
}

void Popup::add(const List<Str> & in)
{
  for (uint i = 0; i < in.size(); ++i)
    menu_add(in[i]);
}

void Popup::draw()
{
  Frame::draw();
  
  if (_icon)
    _icon->draw(x() + w() / 2 - _icon->w() / 2, y() + h() / 2 - _icon->h() / 2);

  if (Fl::focus() == this) draw_focus();
}
  
void Popup::dirty()
{
  Frame::dirty();

  _widget->textfont(text_font());
  _widget->textsize(style::style()->font_size());
  _widget->textcolor(text_color());
  
  _size_hint = style::style()->tool_icon_size() + style::style()->margin() * 2;
}

void Popup::resize(int x, int y, int w, int h)
{
  Frame::resize(x, y, w, h);
  _widget->resize(x, y, w, h);
}

void Popup::widget_callback()
{
  sig.emit(_widget->value());
}

void Popup::menu(Fl_Menu_Item * in)
{
  _widget->clear();
  _widget->copy(in);
}

}
}
}
