/*------------------------------------------------------------------------------
 lib/djv_ui/num.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "num.h"
#include <djv_ui/layout_row.h>
#include <djv_ui/button.h>
#include <FL/Fl.H>

namespace djv {
namespace ui {
namespace num {

Int_Base::Int_Base()
{
  range(-1000000, 1000000);
  inc(1, 10);
}

Float_Base::Float_Base() :
  _precision(0)
{
  range(-1000000.0, 1000000.0);
  inc(0.1, 1.0);
}

void Float_Base::precision(int in)
{
  if (in == _precision) return;
  _precision = in;
  set(get());
  value_update();
}

Inc::Inc()
{
  // Widgets.
  
  _inc_button = new button::Tool("tool_inc", 0);
  _inc_button->type(button::Tool::REPEAT);
  _dec_button = new button::Tool("tool_dec", 0);
  _dec_button->type(button::Tool::REPEAT);
  
  // Layout.
  
  size_policy(FIXED, STRETCH);
  
  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(0);
  layout->margin(0);
  _inc_button->size_policy(FIXED, STRETCH);
  layout->add(_inc_button);
  _dec_button->size_policy(FIXED, STRETCH);
  layout->add(_dec_button);

  // Callbacks.
  
  _inc_button->sig.set(this, inc_callback);
  _dec_button->sig.set(this, dec_callback);
}

void Inc::inc_callback(bool) { inc_sig.emit(true); }
void Inc::dec_callback(bool) { dec_sig.emit(true); }

}
}
}
