/*------------------------------------------------------------------------------
 lib/djv_ui/int.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "num_int.h"
#include <djv_ui/layout_row.h>
#include <djv_ui/button.h>
#include <djv_ui/text.h>
#include <djv_ui/slider.h>
#include <FL/Fl.H>

namespace djv {
namespace ui {
namespace num {

Int_Edit::Int_Edit()
{
  // Widgets.
  
  _widget = new text::Edit;
  _widget->size_str("00000");
  
  Inc * inc_widget = new Inc;
  
  // Layout.
  
  size_policy(FIXED, FIXED);
  align_policy(LEFT, CENTER);
  
  layout::Horizontal * layout = new layout::Horizontal(this);
  layout->spacing(2);
  layout->margin(0);
  layout->add(_widget);
  layout->stretch(_widget);
  layout->add(inc_widget);
  
  // Initialize.
  
  value_update();
  
  // Callbacks.
  
  _widget->sig.set(this, widget_callback);
  inc_widget->inc_sig.set(this, inc_callback);
  inc_widget->dec_sig.set(this, dec_callback);
}

int Int_Edit::handle(int in)
{
  if (Int_Base::handle(in)) return 1;
  return widget::Widget::handle(in);
}

layout::Item * Int_Edit::group_align() { return _widget->group_align(); }

void Int_Edit::widget_callback(const Str & in)
{
  Int_Base::set(str::str_to_int(in));
  
  value_update();
}

void Int_Edit::inc_callback(bool) { Int_Base::inc_small_action(); }
void Int_Edit::dec_callback(bool) { Int_Base::dec_small_action(); }

void Int_Edit::value_update()
{
  callbacks(false);
  _widget->set(str::int_to_str(get()));
  callbacks(true);
}

Int_Display::Int_Display()
{
  // Widgets.
  
  _widget = new text::Display;
  _widget->size_str("00000");
  
  // Layout.
  
  size_policy(FIXED, FIXED);
  align_policy(LEFT, CENTER);
  
  layout::Horizontal * layout = new layout::Horizontal(this);
  layout->spacing(0);
  layout->margin(0);
  layout->add(_widget);
}

void Int_Display::set(int in)
{
  _widget->set(str::int_to_str(in));
}

Int_Slider::Int_Slider() :
  _label(0)
{
  // Widgets.
  
  _widget = new text::Edit;
  _widget->size_str("00000");
  
  Inc * inc_widget = new Inc;
  
  _slider = new slider::Int;
  
  _default_widget = new button::Tool("tool_x");
  _default_widget->tooltip("Reset value.");
  _default_widget->deactivate();
  _default_widget->hide();
  
  // Layout.
  
  size_policy(STRETCH, FIXED);
  
  _layout = new layout::Horizontal(this);
  _layout->spacing(2);
  _layout->margin(0);
  _layout->add(_widget);
  _layout->add(inc_widget);
  _layout->add(_slider);
  _layout->stretch(_slider);
  _layout->add(_default_widget);

  // Initialize.
  
  range(0, 100);
  
  range_update();
  value_update();

  // Callbacks.
  
  _widget->sig.set(this, widget_callback);
  inc_widget->inc_sig.set(this, inc_callback);
  inc_widget->dec_sig.set(this, dec_callback);
  _slider->sig.set(this, slider_callback);
  _default_widget->sig.set(this, default_callback);
}

void Int_Slider::default_valid(bool in)
{
  Int_Base::default_valid(in);
  if (in) _default_widget->show();
  else _default_widget->hide();
  update();
}

void Int_Slider::label(widget::Widget * in)
{
  if (_label) _layout->del(_label);
  _label = in;
  if (_label) _layout->insert(_label, 0);
  _layout->stretch(_slider);
  update();
}

int Int_Slider::handle(int in)
{
  if (Int_Base::handle(in)) return 1;
  return widget::Widget::handle(in);
}

layout::Item * Int_Slider::group_align() { return _widget->group_align(); }

void Int_Slider::widget_callback(const Str & in)
{
  Int_Base::set(str::str_to_int(in));
  value_update();
}

void Int_Slider::inc_callback(bool)
{
  inc_small_action();
}

void Int_Slider::dec_callback(bool)
{
  dec_small_action();
}

void Int_Slider::slider_callback(int in)
{
  Int_Base::set(in);
}

void Int_Slider::default_callback(bool)
{
  Int_Base::set(default_value());
}

void Int_Slider::value_update()
{
  callbacks(false);
  
  const int value = get();
  
  _widget->set(str::int_to_str(value));
  _slider->set(value);
  
  if (default_valid() && value != default_value()) _default_widget->activate();
  else _default_widget->deactivate();

  callbacks(true);
}

void Int_Slider::range_update()
{
  callbacks(false);
  
  _slider->range(min(), max());
  _slider->inc(inc_small(), inc_large());

  callbacks(true);
}

}
}
}
