/*------------------------------------------------------------------------------
 plugin/djv_png/plugin.h

 Portable Network Graphics (PNG) Image I/O

 Requires:
 * libpng - http://www.libpng.org

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_PNG_PLUGIN_H
#define DJV_PNG_PLUGIN_H

#include <djv_image/io.h>
#include <png.h>

using namespace djv;

// Information.

static const Str plugin_name = "PNG";
static const List<Str> extension_list = List<Str>() << ".png";

// Errors.

struct Png_Error {
  char msg [str::cstr_size];
};

extern "C" {
void djv_png_error(png_structp, png_const_charp);
void djv_png_warning(png_structp, png_const_charp);
}

// Load.

class Load : public image::io::Load {
public:

  Load();
  ~Load();

  Str name() const { return ::plugin_name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  plugin::Plugin * copy() const { return new Load; }
  
  void open(const File &, image::io::Info *) throw (Error);
  const image::Image * get(int frame, int proxy) throw (Error);

private:

  void _open(const File &, image::io::Info *) throw (Error);
  void _close();

  File _file;
  FILE * _f;
  png_structp _png;
  png_infop _png_info, _png_info_end;
  Png_Error _png_error;
  image::Image _image, _image_tmp;
};

// Save.

class Save : public image::io::Save {
public:

  Save();
  ~Save();
  
  Str name() const { return ::plugin_name; }
  const List<Str> & extension_list() const { return ::extension_list; }
  
  plugin::Plugin * copy() const { return new Save; }
  
  void open(const File &, const image::io::Info &) throw (Error);
  void set(const image::Image &, int frame) throw (Error);

private:

  void _open(const File &, const image::tag::Tag &) throw (Error);
  void _close();
  
  File _file;
  FILE * _f;
  png_structp _png;
  png_infop _png_info;
  Png_Error _png_error;
  image::Image _image;
};

#endif

