/*------------------------------------------------------------------------------
 plugin/djv_quicktime_apple/ui.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "ui.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/choice.h>

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_ui_image_save()
  { return new Save_Widget; }
}

Save_Widget::Save_Widget() :
  _plugin(0)
{
  // Widgets.
  
  ui::group::Box * codec_group = new ui::group::Box;
  _codec_widget = new ui::choice::Widget(codec_label());
  _codec_widget->label(new ui::label::Widget("Codec:"));
  codec_group->label_widget(_codec_widget);
  ui::label::Text * codec_text = new ui::label::Text(
    "File compression codec.");
  
  ui::group::Box * quality_group = new ui::group::Box;
  _quality_widget = new ui::choice::Widget(quality_label());
  _quality_widget->label(new ui::label::Widget("Quality:"));
  quality_group->label_widget(_quality_widget);
  ui::label::Text * quality_text = new ui::label::Text(
    "File compression quality.");

  // Layout.
  
  ui::layout::group(List<ui::layout::Item *>() << _codec_widget <<
    _quality_widget);
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  layout->spacing(ui::style::style()->spacing_large());

  layout->add(codec_group);
  ui::layout::Vertical * group_layout = new ui::layout::Vertical(codec_group);
  group_layout->add(codec_text);
  
  layout->add(quality_group);
  group_layout = new ui::layout::Vertical(quality_group);
  group_layout->add(quality_text);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _codec_widget->sig.set(this, codec_callback);
  _quality_widget->sig.set(this, quality_callback);
}

void Save_Widget::set(image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Save_Widget::codec_callback(int in)
{
  if (! _plugin) return;
  _codec = CODEC(in);
  plugin_update();
  sig.emit(_plugin);
}

void Save_Widget::quality_callback(int in)
{
  if (! _plugin) return;
  _quality = QUALITY(in);
  plugin_update();
  sig.emit(_plugin);
}

void Save_Widget::plugin_update()
{
  callbacks(false);

  if (_plugin) {
    List<Str> tmp;
    tmp = str::serialize(_codec, codec_label());
    _plugin->option(_plugin->option_list()[::Save::OPTION_CODEC], &tmp);
    tmp = str::serialize(_quality, quality_label());
    _plugin->option(_plugin->option_list()[::Save::OPTION_QUALITY], &tmp);
  }

  callbacks(true);
}

void Save_Widget::widget_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[::Save::OPTION_CODEC]);
    str::serialize(&tmp, &_codec, codec_label());
    tmp = _plugin->option(_plugin->option_list()[::Save::OPTION_QUALITY]);
    str::serialize(&tmp, &_quality, quality_label());
  }
  
  _codec_widget->set(_codec);
  _quality_widget->set(_quality);
  
  callbacks(true);
}
